﻿using System;
using System.Windows;
using System.Windows.Media.Animation;

namespace TestProject
{
    /// <summary>
    /// 원 애니메이션
    /// </summary>
    public class CircleAnimation : DoubleAnimationBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 방향 타입 - DirectionType

        /// <summary>
        /// 방향 타입
        /// </summary>
        public enum DirectionType
        {
            /// <summary>
            /// X 방향
            /// </summary>
            XDirection,

            /// <summary>
            /// Y 방향
            /// </summary>
            YDirection
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 반경 속성 - DependencyProperty

        /// <summary>
        /// 반경 속성
        /// </summary>
        public static readonly DependencyProperty RadiusProperty = DependencyProperty.Register
        (
            "Radius",
            typeof(double),
            typeof(CircleAnimation),
            new PropertyMetadata((double) 10)
        );

        #endregion
        #region 방향 속성 - DirectionProperty

        /// <summary>
        /// 방향 속성
        /// </summary>
        public static readonly DependencyProperty DirectionProperty = DependencyProperty.Register
        (
            "Direction",
            typeof(DirectionType),
            typeof(CircleAnimation),
            new PropertyMetadata(DirectionType.XDirection)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 반경 - Radius

        /// <summary>
        /// 반경
        /// </summary>
        public double Radius
        {
            get
            {
                return (double)GetValue(RadiusProperty);
            }
            set
            {
                if(value > 0.0)
                {
                    SetValue(RadiusProperty, value);
                }
                else
                {
                    throw new ArgumentException($"허용되지 않는 반경입니다 : {value}");
                }
            }
        }

        #endregion
        #region 방향 - Direction

        /// <summary>
        /// 방향
        /// </summary>
        public DirectionType Direction
        {
            get
            {
                return (DirectionType)GetValue(DirectionProperty);
            }
            set
            {
                SetValue(DirectionProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 인스턴스 생성하기 (코어) - CreateInstanceCore()

        /// <summary>
        /// 인스턴스 생성하기 (코어)
        /// </summary>
        /// <returns>생성 인스턴스</returns>
        protected override Freezable CreateInstanceCore() => new CircleAnimation();

        #endregion
        #region 현재 값 구하기 (코어) - GetCurrentValueCore(defaultSourceValue, defaultTargetValue, clock)

        /// <summary>
        /// 현재 값 구하기 (코어)
        /// </summary>
        /// <param name="defaultSourceValue">디폴트 소스 값</param>
        /// <param name="defaultTargetValue">디폴트 타겟 값</param>
        /// <param name="clock">애니메이션 클럭</param>
        /// <returns>현재 값</returns>
        protected override double GetCurrentValueCore(double defaultSourceValue, double defaultTargetValue, AnimationClock clock)
        {
            double time = clock.CurrentProgress.Value;

            double value = Direction == DirectionType.XDirection ? Math.Cos(2 * Math.PI * time) : Math.Sin(2 * Math.PI * time);

            return value * Radius + defaultSourceValue;
        }

        #endregion
    }
}